/* SPDX-License-Identifier: BSD-3-Clause */

#ifndef __SO_RINGBUFFER_H__
#define __SO_RINGBUFFER_H__

#include <sys/types.h>
#include <string.h>
#include <pthread.h>

typedef struct so_ring_buffer_t {
	char *data;

	size_t read_pos;
	size_t write_pos;

	size_t len;
	size_t cap;

	/* TODO: Add syncronization primitives */
	pthread_mutex_t lock;
	pthread_cond_t full_cond;
	pthread_cond_t empty_cond;
	int stop;
} so_ring_buffer_t;

// Lista simplu inlantuita folosita pentru a tine timestamp-urile
typedef struct list_node {
    unsigned long timestamp;
    struct list_node *next;
} list_node;

typedef struct timestamp_list {
    list_node *head;
	list_node *tail;
    pthread_mutex_t lock;
	pthread_cond_t wait_packet;
	int stop;
} timestamp_list;

// Functii pentru ring buffer
int     ring_buffer_init(so_ring_buffer_t *rb, size_t cap);
ssize_t ring_buffer_enqueue(so_ring_buffer_t *rb, void *data, size_t size);
ssize_t ring_buffer_dequeue(so_ring_buffer_t *rb, void *data, size_t size);
void    ring_buffer_destroy(so_ring_buffer_t *rb);
void    ring_buffer_stop(so_ring_buffer_t *rb);

// Functii pentru lista simplu inlantuita
void init_list();
void add_to_list(unsigned long timestamp);
void remove_from_list();

#endif /* __SO_RINGBUFFER_H__ */
